
/*
  Copyright 1995 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/

// This is the main CGI program which extracts the requested variable from a
// netCDF data file. It generates the DDS in-memory from both the netCDF
// file and external file and uses it to locate the variable to process.
//
// ReZa 12/29/94

// $Log: cdf_dods.cc,v $
// Revision 1.1  1996/06/10 20:34:13  jimg
// Added.
//
// Revision 1.7  1995/07/09  21:33:59  jimg
// Added copyright notice.
//
// Revision 1.6  1995/06/29  15:42:09  jimg
// Fixed nc_dods.cc
//
// Revision 1.5  1995/06/27  21:07:32  jimg
// Fixed up the error reporting. Added/corrected some comments.
//
// Revision 1.4  1995/06/23  15:50:35  jimg
// Changed from a CGI to a simple UNIX filter.
//
// Revision 1.3  1995/04/28  20:40:01  reza
// Moved hyperslab access to the server side (passed as a constraint argument).
//
// Revision 1.2  1995/03/16  16:44:06  reza
// Added TEST directives for data server debugging.
//
// Revision 1.1  1995/02/10  04:49:24  reza
// First version of Data Server for netCDF files.
//

static char rcsid[]={"$Id: cdf_dods.cc,v 1.1 1996/06/10 20:34:13 jimg Exp $"};

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <iostream.h>

#include "netio.h"
#include "DDS.h"
#include "CDFArray.h"
#include "CDFGrid.h"

int 
main(int argc, char * argv[])
{
    static char ExtFile[128], Msgt[255];
    extern bool read_descriptors(DDS &das, char *filename);
    DDS dds;

    set_mime_binary(); 

    if(argc < 3) {
	ErrMsgT("nc_dods: Usage: nc_dods <datafile> <variable>");
        exit(1);
    }

    // Read the CDF file dataset descriptor in memory

    if(!read_descriptors(dds, argv[1])){
	ErrMsgT("cdf_dods: Could not read the data descriptor information");
	exit(1);
    }

    // if external descriptor file exist, read it ...
    // This file must be in the same directory that 
    // the netCDF file is located. Its name must be the
    // netCDF file name with the addition of .dds

    sprintf (ExtFile, "%s.dds", argv[1]); 
    FILE *in = fopen(ExtFile, "r");
 
    if (in) {
	int status = dds.parse(in);

	if(!status) {
	    sprintf (Msgt, "cdf_dods: Parse error in external file %s", 
		     ExtFile);
	    ErrMsgT(Msgt);
	    exit(1);
	}
	fclose(in);
    }

    // Read the variable, serialize it and send it over the network
    for( Pix p = dds.first_var(); p; dds.next_var(p) ) {
	String vartype = dds.var(p)->get_var_type();

	// find and process only vaild types
	if ((vartype == "Array") || (vartype == "Grid")) {
	    if ( dds.var(p)->get_var_name() == argv[2] ) {  // is this the
							    // variable? 
		//Read it into buffer
		String Fname = argv[1];
		String Vname = argv[2];
		String Constraint = argv[3];
		if ( dds.var(p)->read(Fname, Vname, Constraint)) 
		    if ( !(dds.var(p)->serialize(true)) ) {
			sprintf (Msgt,
				 "cdf_dods: Error serializing variable %s",
				 argv[2]); 
			ErrMsgT(Msgt);
			exit(1);
		    }
		break;
	    }					   
	}
    }
}


