/*-------------------------------------------------------------------------
 * Copyright (C) 1999	National Center for Supercomputing Applications.
 *			All rights reserved.
 *
 *-------------------------------------------------------------------------
 */

// This is the main CGI program which extracts DDS class descriptors and translate
// data to DODS from a
// hdf5 data file. 


#include "h5dds.h"
#include "InternalErr.h"

extern "C" hid_t get_fileid(const char *);

int
main(int argc, char *argv[])
{
    hid_t file1;
    DODSFilter df(argc, argv);

    try {
	if (!df.OK()) {
	    df.print_usage();
	    return 1;
	}

	if (df.version()) {
	    df.send_version_info();
	    return 0;
	}

	file1 = get_fileid(df.get_dataset_name().c_str());
	if (file1 < 0) {
	    string msg =
		"unable to open hdf5 file " + df.get_dataset_name();
	    throw Error(no_such_file, msg);
	}

	DDS dds;

	depth_first(file1, "/", dds, df.get_dataset_name().c_str());

	df.send_data(dds, stdout);

	if (H5Fclose(file1) < 0)
	    throw InternalErr(__FILE__, __LINE__,
			      "Could not close the HDF5 file: "
			      + df.get_dataset_name());
    }

    catch(Error & e) {
	(void) H5Fclose(file1);
	set_mime_text(cout, dods_error, df.get_cgi_version());
	e.print(cout);
	return 1;
    }

    return 0;
}
